import {
  SubstrateQueryConfig,
  ExtrinsicConfig,
  ContractConfig,
} from '@moonbeam-network/xcm-builder';
import { TransferConfig, AssetConfig } from '@moonbeam-network/xcm-config';
import {
  AnyParachain,
  Asset,
  AssetAmount,
  ChainAssetId,
  AnyChain,
  Ecosystem,
} from '@moonbeam-network/xcm-types';
import { Signer as Signer$1 } from 'ethers';
import { ApiPromise } from '@polkadot/api';
import { Signer } from '@polkadot/api/types';
import { IKeyringPair } from '@polkadot/types/types';

declare class PolkadotService {
  #private;
  readonly api: ApiPromise;
  readonly chain: AnyParachain;
  constructor(api: ApiPromise, chain: AnyParachain);
  static create(chain: AnyParachain): Promise<PolkadotService>;
  static createMulti(chains: AnyParachain[]): Promise<PolkadotService[]>;
  get decimals(): number;
  get asset(): Asset;
  get existentialDeposit(): AssetAmount;
  getAssetMeta(asset: ChainAssetId): Promise<
    | {
        symbol: string;
        decimals: number;
      }
    | undefined
  >;
  getAssetDecimals(asset: Asset): Promise<number>;
  query(config: SubstrateQueryConfig): Promise<bigint>;
  getFee(account: string, config: ExtrinsicConfig): Promise<bigint>;
  transfer(
    account: string,
    config: ExtrinsicConfig,
    signer: Signer | IKeyringPair,
  ): Promise<string>;
}

interface Signers {
  ethersSigner: Signer$1;
  polkadotSigner: Signer | IKeyringPair;
}
interface TransferData {
  destination: DestinationChainTransferData;
  getEstimate(amount: number | string): AssetAmount;
  isSwapPossible: boolean;
  max: AssetAmount;
  min: AssetAmount;
  source: SourceChainTransferData;
  swap(): Promise<TransferData | undefined>;
  transfer(amount: bigint | number | string): Promise<string>;
}
interface SourceChainTransferData extends ChainTransferData {
  feeBalance: AssetAmount;
  max: AssetAmount;
}
interface DestinationChainTransferData extends ChainTransferData {}
interface ChainTransferData {
  balance: AssetAmount;
  chain: AnyChain;
  existentialDeposit: AssetAmount;
  fee: AssetAmount;
  min: AssetAmount;
}

interface GetSourceDataParams {
  transferConfig: TransferConfig;
  destinationAddress: string;
  destinationFee: AssetAmount;
  ethersSigner?: Signer$1;
  polkadot: PolkadotService;
  sourceAddress: string;
}
declare function getSourceData({
  transferConfig,
  destinationAddress,
  destinationFee,
  ethersSigner,
  polkadot,
  sourceAddress,
}: GetSourceDataParams): Promise<SourceChainTransferData>;
interface GetBalancesParams {
  address: string;
  balance: bigint;
  config: AssetConfig;
  polkadot: PolkadotService;
}
declare function getFeeBalances({
  address,
  balance,
  config,
  polkadot,
}: GetBalancesParams): Promise<bigint>;
interface GetFeeParams {
  balance: bigint;
  contract?: ContractConfig;
  decimals: number;
  ethersSigner?: Signer$1;
  extrinsic?: ExtrinsicConfig;
  polkadot: PolkadotService;
  sourceAddress: string;
}
declare function getFee({
  balance,
  contract,
  decimals,
  ethersSigner,
  extrinsic,
  polkadot,
  sourceAddress,
}: GetFeeParams): Promise<bigint>;
declare function getContractFee(
  balance: bigint,
  config: ContractConfig,
  decimals: number,
  ethersSigner: Signer$1,
): Promise<bigint>;
declare function getExtrinsicFee(
  balance: bigint,
  extrinsic: ExtrinsicConfig,
  polkadot: PolkadotService,
  sourceAddress: string,
): Promise<bigint>;
interface GetMaxParams {
  balanceAmount: AssetAmount;
  existentialDeposit: AssetAmount;
  feeAmount: AssetAmount;
  minAmount: AssetAmount;
}
declare function getMax({
  balanceAmount,
  existentialDeposit,
  feeAmount,
  minAmount,
}: GetMaxParams): AssetAmount;

interface SdkOptions extends Partial<Signers> {}
declare function Sdk(options?: SdkOptions): {
  assets(ecosystem?: Ecosystem): {
    assets: Asset[];
    asset(keyOrAsset: string | Asset): {
      sourceChains: AnyChain[];
      source(keyOrChain: string | AnyChain): {
        destinationChains: AnyChain[];
        destination(destKeyOrChain: string | AnyChain): {
          accounts(
            sourceAddress: string,
            destinationAddress: string,
            signers?: Partial<Signers>,
          ): Promise<TransferData>;
        };
      };
    };
  };
  getTransferData({
    destinationAddress,
    destinationKeyOrChain,
    ethersSigner,
    keyOrAsset,
    polkadotSigner,
    sourceAddress,
    sourceKeyOrChain,
  }: SdkTransferParams): Promise<TransferData>;
};
interface SdkTransferParams extends Partial<Signers> {
  destinationAddress: string;
  destinationKeyOrChain: string | AnyChain;
  keyOrAsset: string | Asset;
  sourceAddress: string;
  sourceKeyOrChain: string | AnyChain;
}

export {
  ChainTransferData,
  DestinationChainTransferData,
  GetBalancesParams,
  GetFeeParams,
  GetMaxParams,
  GetSourceDataParams,
  Sdk,
  SdkOptions,
  SdkTransferParams,
  Signers,
  SourceChainTransferData,
  TransferData,
  getContractFee,
  getExtrinsicFee,
  getFee,
  getFeeBalances,
  getMax,
  getSourceData,
};
